@@ -8,7 +8,7 @@ head(TravelChoice)
 str(TravelChoice)
 
 ## -----------------------------------------------------------------------------
-exp_8.4 <- Discrete_CM(
+exp_8.4 <- discrete_cm(
   formula = choice ~ hinc + gc + invt,
   case_id = "indv",
   alternatives = "mode",
@@ -19,7 +19,7 @@ exp_8.4 <- Discrete_CM(
 summary(exp_8.4)
 
 ## -----------------------------------------------------------------------------
-(constant_model <- Discrete_CM(
+(constant_model <- discrete_cm(
   formula = choice ~ 1 ,
   case_id = "indv",
   alternatives = "mode",
@@ -28,7 +28,7 @@ summary(exp_8.4)
   cdf = "logistic"
 ))
 
-(car_0 <- Discrete_CM(
+(car_0 <- discrete_cm(
   formula = choice ~ hinc[air] + psize[air] + gc + ttme,
   case_id = "indv",
   alternatives = "mode",
@@ -39,7 +39,7 @@ summary(exp_8.4)
 ))
 
 ## -----------------------------------------------------------------------------
-mod_1 <- Discrete_CM(
+mod_1 <- discrete_cm(
   formula = choice ~ hinc[air] + psize[air] + gc + ttme,
   case_id = "indv",
   alternatives = "mode",
@@ -51,38 +51,38 @@ mod_1 <- Discrete_CM(
 logLik(mod_1)
 
 ## -----------------------------------------------------------------------------
-mod_2 <- Discrete_CM(
+mod_2 <- discrete_cm(
   formula = choice ~ hinc[air] + psize[air] + gc + ttme,
   case_id = "indv",
   alternatives = "mode",
   reference = "bus",
   alternative_specific = c("gc", "ttme"),
   data = TravelChoice,
-  cdf = student.glmcat(df = 30)
+  cdf = list("student",30)
 )
 logLik(mod_2)
 
 ## -----------------------------------------------------------------------------
-mod_3 <- Discrete_CM(
+mod_3 <- discrete_cm(
   formula = choice ~ hinc[air] + psize[air] + gc + ttme,
   case_id = "indv",
   alternatives = "mode",
   reference = "car",
   alternative_specific = c("gc", "ttme"),
   data = TravelChoice,
-  cdf = student.glmcat(df = 0.2)
+  cdf = list("student",0.2)
 )
 logLik(mod_3)
 
 ## -----------------------------------------------------------------------------
-mod_4 <- Discrete_CM(
+mod_4 <- discrete_cm(
   formula = choice ~ hinc[air] + psize[air] + gc + ttme,
   case_id = "indv",
   alternatives = "mode",
   reference = "train",
   alternative_specific = c("gc", "ttme"),
   data = TravelChoice,
-  cdf = student.glmcat(df = 1.35)
+  cdf = list("student",1.35)
 )
 logLik(mod_4)
 
